@@ -108,7 +108,7 @@ Mojo::Pg::Migrations - Migrations
   use Mojo::Pg::Migrations;
 
   my $migrations = Mojo::Pg::Migrations->new(pg => $pg);
-  $migrations->from_file('/Users/sri/migrations.sql')->migrate;
+  $migrations->from_file('/home/sri/migrations.sql')->migrate;
 
 =head1 DESCRIPTION
 
@@ -184,7 +184,7 @@ L</"name">.
 
 =head2 from_file
 
-  $migrations = $migrations->from_file('/Users/sri/migrations.sql');
+  $migrations = $migrations->from_file('/home/sri/migrations.sql');
 
 Extract migrations from a file.
 
